//loadram.c: main source code, uploads binary file to ZRAM
//Copyright 2006 Jeroen Domburg <s1fwx@jeroen.ietsmet.nl>
//This software is licensed under the GNU GPL.

#include "loadram.h"

int main(int argc, char** argv) {
    static struct usb_dev_handle *h;
    int r;
    FILE *f;
    struct stat fstat;
    char *membuff;
    size_t nobytes;
    CMD c;


    //0x3400 is an ECC checking scratchpad, so we should be able to safely use it
    unsigned int toaddr=0x3400;

    if (argc!=2) {
		printf("Usage: %s file.bin\n",argv[0]);
		exit(0);
    }

    memset(&c,0,sizeof(c));

    printf("Reading & uploading program %s...\n",argv[1]);
    r=stat(argv[1],&fstat);
    if (r!=0) {
	perror("Statting input file");
	exit(1);
    }
    nobytes=fstat.st_size;
//Pad to a multiple of 256 bytes.
//Hw can prolly handle less granular transfers, this is just for the easy-factor.
    nobytes+=255; nobytes=nobytes&0xff00; 
    printf("%li bytes; padded to %i (0x%x).\n",fstat.st_size,nobytes,nobytes);

//Alloc buffer & read
    membuff=malloc(nobytes);
    f=fopen(argv[1], "rb");
    if (!f) {
	perror("Reading input file");
	exit(1);
    }
    r=fread(membuff,1,fstat.st_size,f);
    if (r<fstat.st_size) {
	perror("Short read at input file");
    }
    printf("%i bytes read.\n",nobytes);
    fclose(f);

    //Open libusb handle to device
    h = device_open('a');
    if (h==NULL) {
#ifdef LINUX
	printf("\nNo device detected.\n");
#else
	printf("\nCan't find any libusb Device, or there were errors either claiming or configuring the interface....Quitting\n");
#endif
	exit(1);
    }

//The following code contains quite a few 'magic numbers'. These are gleaned
//from usb logs from a firmware update by the original Windows-programs.
//I'm not sure if all the magic really is necessary, but let's keep it until
//we're sure.
    //Upload program to RAM
    c.mode = 0x0;
    c.cmd = 0x3005+(toaddr<<16);
    c.size = (uint32)nobytes; 
    c.unkwn0x13=(8<<16)+0xcb03; //page 0
    c.unkwn0x17=(uint32)(0x4100+(nobytes>>8));
    device_send_cmd(h, &c, (void *)membuff, (uint32)nobytes);
//and run
    printf("Running the program...\n");
    c.mode = 0xf8;
    c.cmd = 0xa0000020+(toaddr<<8);
    c.size = 0; 
    c.unkwn0x13=0x80000;
    c.unkwn0x17=0xe57000;
    device_send_cmd(h, &c, (void *)membuff, 0);

    printf("Done.\n");
    device_close(h);
    return(0);
}
